@@ -17,9 +17,6 @@ class CreateTorrentJob(base.JobBase):
     """
     Create torrent file
 
-    :param content_path: Path to torrent content
-    :param str tracker: :class:`~.trackers.base.TrackerBase` subclass
-
     This job adds the following signals to the :attr:`~.JobBase.signal`
     attribute:
 
@@ -34,10 +31,16 @@ class CreateTorrentJob(base.JobBase):
 
     @property
     def cache_id(self):
-        """Use tracker name for cache ID"""
-        return self._tracker.name
+        """Final segment of `content_path` and tracker :attr:`~.TrackerBase.name`"""
+        return (fs.basename(self._content_path), self._tracker.name)
 
     def initialize(self, *, tracker, content_path):
+        """
+        Set internal state
+
+        :param TrackerBase tracker: Return value of :func:`.trackers.tracker`
+        :param content_path: Path to file or directory
+        """
         self._tracker = tracker
         self._content_path = content_path
         self._torrent_path = os.path.join(
@@ -45,7 +48,29 @@ class CreateTorrentJob(base.JobBase):
             f'{fs.basename(content_path)}.{tracker.name.lower()}.torrent',
         )
         self.signal.add('progress_update')
-        self._file_tree = ''
+        self._torrent_process = None
+
+    def execute(self):
+        """Get announce URL from `tracker`, then execute torrent creation subprocess"""
+        self.add_task(self._get_announce_url())
+
+    async def _get_announce_url(self):
+        self.info = 'Getting announce URL...'
+        try:
+            await self._tracker.login()
+            announce_url = await self._tracker.get_announce_url()
+        except errors.RequestError as e:
+            self.error(e, finish=True)
+        else:
+            self._create_torrent_process(announce_url)
+        finally:
+            self.info = ''
+            try:
+                await self._tracker.logout()
+            except errors.RequestError as e:
+                self.warn(e)
+
+    def _create_torrent_process(self, announce_url):
         self._torrent_process = daemon.DaemonProcess(
             name=self.name,
             target=_torrent_process,
@@ -53,42 +78,42 @@ class CreateTorrentJob(base.JobBase):
                 'content_path' : self._content_path,
                 'torrent_path' : self._torrent_path,
                 'overwrite'    : self.ignore_cache,
-                'announce'     : tracker.config['announce'],
-                'source'       : tracker.config['source'],
-                'exclude'      : tracker.config['exclude'],
+                'announce'     : announce_url,
+                'source'       : self._tracker.config['source'],
+                'exclude'      : self._tracker.config['exclude'],
             },
-            init_callback=self.handle_file_tree,
-            info_callback=self.handle_progress_update,
-            error_callback=self.error,
-            finished_callback=self.handle_torrent_created,
+            init_callback=self._handle_file_tree,
+            info_callback=self._handle_progress_update,
+            error_callback=self._handle_error,
+            result_callback=self._handle_torrent_created,
+            finished_callback=self.finish,
         )
-
-    def execute(self):
         self._torrent_process.start()
 
     def finish(self):
-        self._torrent_process.stop()
+        """Terminate torrent creation subprocess and finish"""
+        if self._torrent_process:
+            self._torrent_process.stop()
         super().finish()
 
-    async def wait(self):
-        await self._torrent_process.join()
-        await super().wait()
+    def _handle_file_tree(self, file_tree):
+        self.info = fs.file_tree(file_tree)
 
-    def handle_file_tree(self, file_tree):
-        self._file_tree = fs.file_tree(file_tree)
-
-    @property
-    def info(self):
-        return self._file_tree
-
-    def handle_progress_update(self, percent_done):
+    def _handle_progress_update(self, percent_done):
         self.signal.emit('progress_update', percent_done)
 
-    def handle_torrent_created(self, torrent_path=None):
+    def _handle_torrent_created(self, torrent_path=None):
         _log.debug('Torrent created: %r', torrent_path)
         if torrent_path:
             self.send(torrent_path)
-        self.finish()
+
+    def _handle_error(self, error):
+        if isinstance(error, BaseException):
+            _log.debug('CreateTorrentJob: Raising %r', error)
+            self.exception(error)
+        else:
+            _log.debug('CreateTorrentJob: Reporting %r', error)
+            self.error(error)
 
 
 def _torrent_process(output_queue, input_queue, *args, **kwargs):
@@ -122,14 +147,6 @@ class AddTorrentJob(base.QueueJobBase):
     """
     Add torrent(s) to a BitTorrent client
 
-    :param client: Return value of :func:`.utils.btclients.client`
-    :param download_path: Path to the torrent's content or `None` to use the
-        client's default path
-    :param enqueue: Sequence of torrent file paths to add
-
-    If `enqueue` is given and not empty, this job is finished as soon as its
-    last item is added.
-
     This job adds the following signals to the :attr:`~.JobBase.signal`
     attribute:
 
@@ -139,9 +156,8 @@ class AddTorrentJob(base.QueueJobBase):
 
         ``added``
             Emitted when the torrent was added successfully. Registered
-            callbacks get the some kind of ID that uniquely identifies the added
-            torrent for the client. It depends on the `client` object what that
-            is exactly.
+            callbacks get the added torrent's info hash as a positional
+            argument.
     """
 
     name = 'add-torrent'
@@ -149,20 +165,31 @@ class AddTorrentJob(base.QueueJobBase):
     cache_id = None  # Don't cache output
 
     def initialize(self, *, client, download_path=None, enqueue=()):
+        """
+        Set internal state
+
+        :param client: Return value of :func:`.btclients.client`
+        :param download_path: Path to the torrent's download location or `None`
+            to use the client's default path
+        :param enqueue: Sequence of torrent file paths to add
+
+        If `enqueue` is given and not empty, this job is finished as soon as its
+        last item is added.
+        """
         assert isinstance(client, btclients.ClientApiBase), f'Not a ClientApiBase: {client!r}'
         self._client = client
         self._download_path = download_path
         self.signal.add('adding')
         self.signal.add('added')
-        self._info = ''
-        self.signal.register('adding', lambda tp: setattr(self, '_info', f'Adding {fs.basename(tp)}'))
-        self.signal.register('added', lambda _: setattr(self, '_info', ''))
-        self.signal.register('finished', lambda: setattr(self, '_info', ''))
-        self.signal.register('error', lambda _: setattr(self, '_info', ''))
+        self.signal.register('adding', lambda tp: setattr(self, 'info', f'Adding {fs.basename(tp)}'))
+        self.signal.register('added', lambda _: setattr(self, 'info', ''))
+        self.signal.register('finished', lambda _: setattr(self, 'info', ''))
+        self.signal.register('error', lambda _: setattr(self, 'info', ''))
 
     MAX_TORRENT_SIZE = 10 * 2**20  # 10 MiB
+    """Upper limit of acceptable size of `.torrent` files"""
 
-    async def _handle_input(self, torrent_path):
+    async def handle_input(self, torrent_path):
         _log.debug('Adding %s to %s', torrent_path, self._client.name)
         self.signal.emit('adding', torrent_path)
 
@@ -171,32 +198,21 @@ class AddTorrentJob(base.QueueJobBase):
             return
 
         try:
-            torrent_id = await self._client.add_torrent(
+            torrent_hash = await self._client.add_torrent(
                 torrent_path=torrent_path,
                 download_path=self._download_path,
             )
         except errors.TorrentError as e:
             self.error(f'Failed to add {torrent_path} to {self._client.name}: {e}')
         else:
-            self.send(torrent_id)
-            self.signal.emit('added', torrent_id)
-
-    @property
-    def info(self):
-        """Status message"""
-        return self._info
+            self.send(torrent_hash)
+            self.signal.emit('added', torrent_hash)
 
 
 class CopyTorrentJob(base.QueueJobBase):
     """
     Copy file(s)
 
-    :param destination: Where to put the file(s)
-    :param enqueue: Sequence of file paths to copy
-
-    If `enqueue` is given and not empty, this job is finished as soon as its
-    last item is copied.
-
     This job adds the following signals to the :attr:`~.JobBase.signal`
     attribute:
 
@@ -215,18 +231,27 @@ class CopyTorrentJob(base.QueueJobBase):
     cache_id = None  # Don't cache output
 
     def initialize(self, *, destination, enqueue=()):
+        """
+        Set internal state
+
+        :param destination: Where to put the torrent(s)
+        :param enqueue: Sequence of file paths to copy
+
+        If `enqueue` is given and not empty, this job is finished as soon as its
+        last item is copied.
+        """
         self._destination = None if not destination else str(destination)
         self.signal.add('copying')
         self.signal.add('copied')
-        self._info = ''
-        self.signal.register('copying', lambda fp: setattr(self, '_info', f'Copying {fs.basename(fp)}'))
-        self.signal.register('copied', lambda _: setattr(self, '_info', ''))
-        self.signal.register('finished', lambda: setattr(self, '_info', ''))
-        self.signal.register('error', lambda _: setattr(self, '_info', ''))
+        self.signal.register('copying', lambda fp: setattr(self, 'info', f'Copying {fs.basename(fp)}'))
+        self.signal.register('copied', lambda _: setattr(self, 'info', ''))
+        self.signal.register('finished', lambda _: setattr(self, 'info', ''))
+        self.signal.register('error', lambda _: setattr(self, 'info', ''))
 
     MAX_FILE_SIZE = 10 * 2**20  # 10 MiB
+    """Upper limit of acceptable file size"""
 
-    async def _handle_input(self, filepath):
+    async def handle_input(self, filepath):
         _log.debug('Copying %s to %s', filepath, self._destination)
 
         if not os.path.exists(filepath):
@@ -252,8 +277,3 @@ class CopyTorrentJob(base.QueueJobBase):
             else:
                 self.send(new_path)
                 self.signal.emit('copied', new_path)
-
-    @property
-    def info(self):
-        """Status message"""
-        return self._info
