@@ -3,12 +3,11 @@ Swiss Army knife
 """
 
 import collections
-import enum
 import importlib
 import inspect
 import itertools
 import os
-import types
+import types as _types
 
 
 def os_family():
@@ -41,34 +40,8 @@ except ImportError:
                 return value
 
 
-class ReleaseType(enum.Enum):
-    '''
-    Enum with the values ``movie``, ``series``, ``season``, ``episode`` and
-    ``unknown``
-
-    Use ``season`` or ``episode`` if it matters, ``series`` otherwise.
-
-    All values are truthy except for ``unknown``.
-    '''
-
-    movie = 'movie'
-    series = 'series'
-    season = 'season'
-    episode = 'episode'
-    unknown = 'unknown'
-
-    def __bool__(self):
-        return self is not self.unknown
-
-    def __str__(self):
-        return str(self.value)
-
-    def __repr__(self):
-        return f'{type(self).__name__}.{self.value}'
-
-
 # https://github.com/tensorflow/tensorflow/blob/master/tensorflow/python/util/lazy_loader.py
-class LazyModule(types.ModuleType):
+class LazyModule(_types.ModuleType):
     """
     Lazily import module to decrease execution time
 
@@ -126,9 +99,10 @@ def submodules(package):
         if not name.startswith('_'):
             if name.endswith('.py'):
                 name = name[:-3]
-            submods.append(
-                importlib.import_module(name=f'.{name}', package=package)
-            )
+            if '.' not in name:
+                submods.append(
+                    importlib.import_module(name=f'.{name}', package=package)
+                )
     return submods
 
 
@@ -140,36 +114,32 @@ def subclasses(basecls, modules):
     :param modules: Modules to search
     :type modules: list of module objects
     """
-    subclses = []
+    subclses = set()
     for mod in modules:
         for name, member in inspect.getmembers(mod):
             if (member is not basecls and
                 isinstance(member, type) and
                 issubclass(member, basecls)):
-                subclses.append(member)
+                subclses.add(member)
     return subclses
 
 
-def closest_number(n, ns):
-    """Return the number from `ns` that is closest to `n`"""
-    # https://stackoverflow.com/a/12141207
-    return min(ns, key=lambda x: abs(x - n))
+def closest_number(n, ns, max=None, default=0):
+    """
+    Return the number from `ns` that is closest to `n`
 
-
-_byte_units = {
-    'PiB': 1024 ** 5,
-    'TiB': 1024 ** 4,
-    'GiB': 1024 ** 3,
-    'MiB': 1024 ** 2,
-    'KiB': 1024,
-}
-
-def pretty_bytes(b):
-    '''Return `b` as human-readable bytes, e.g. "24.3 MiB"'''
-    for unit, bytes in _byte_units.items():
-        if b >= bytes:
-            return f'{b / bytes:.2f} {unit}'
-    return f'{int(b)} B'
+    :param n: Given number
+    :param ns: Sequence of allowed numbers
+    :param max: Remove any item from `ns` that is larger than `max`
+    :param default: Return value in case `ns` is empty
+    """
+    if max is not None:
+        ns_ = tuple(n_ for n_ in ns if n_ <= max)
+        if not ns_:
+            raise ValueError(f'No number equal to or below {max}: {ns}')
+    else:
+        ns_ = ns
+    return min(ns_, key=lambda x: abs(x - n), default=default)
 
 
 class CaseInsensitiveString(str):
@@ -225,6 +195,26 @@ def merge_dicts(a, b, path=()):
     return merged
 
 
-from . import (browser, btclients, configfiles, country, daemon, fs, html,
-               http, imghosts, release, screenshot, signal, subproc, timestamp,
-               torrent, video, webdbs)
+def deduplicate(l, key=None):
+    """
+    Return sequence `l` with all duplicate items removed while maintaining the
+    original order
+
+    :param key: Callable that gets each item and returns a hashable identifier
+        for that item
+    """
+    if key is None:
+        key = lambda k: k
+    seen_keys = set()
+    deduped = []
+    for item in l:
+        k = key(item)
+        if k not in seen_keys:
+            seen_keys.add(k)
+            deduped.append(item)
+    return deduped
+
+
+from . import (browser, btclients, configfiles, daemon, fs, html, http, image,
+               imghosts, iso, release, scene, signal, string, subproc,
+               timestamp, torrent, types, video, webdbs)
