@@ -2,7 +2,7 @@
 API for BitTorrent clients
 """
 
-from .. import subclasses, submodules
+from .. import CaseInsensitiveString, subclasses, submodules
 from .base import ClientApiBase
 
 
@@ -17,7 +17,7 @@ def client(name, **kwargs):
 
     :param str name: Name of the client. A subclass of :class:`.ClientApiBase`
         with the same :attr:`~.ClientApiBase.name` must exist in one of this
-        package's submodules
+        package's submodules.
     :param kwargs: All keyword arguments are passed to the subclass specified by
         `name`
 
@@ -29,3 +29,8 @@ def client(name, **kwargs):
         if client.name == name:
             return client(**kwargs)
     raise ValueError(f'Unsupported client: {name}')
+
+
+def client_names():
+    """Return sequence of valid `name` arguments for :func:`.client`"""
+    return sorted(CaseInsensitiveString(cls.name) for cls in clients())
