@@ -6,8 +6,8 @@ import functools
 import os
 import re
 
-from .. import __project_name__, errors
-from . import LazyModule, os_family, pretty_bytes
+from .. import constants, errors
+from . import LazyModule, os_family, string
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -40,42 +40,99 @@ def assert_dir_usable(path):
 
 
 @functools.lru_cache(maxsize=None)
-def tmpdir():
-    """Return path to existing temporary directory in /tmp/ or similar location"""
-    tmpdir_ = tempfile.mkdtemp()
-    parent_path = os.path.dirname(tmpdir_)
-    tmpdir = os.path.join(parent_path, __project_name__)
-    if os.path.exists(tmpdir):
-        os.rmdir(tmpdir_)
-    else:
-        os.rename(tmpdir_, tmpdir)
-    assert_dir_usable(tmpdir)
-    _log.debug('Using temporary directory: %r', tmpdir)
-    return tmpdir
-
-
-@functools.lru_cache(maxsize=None)
-def projectdir(content_path):
+def projectdir(content_path, base=None):
     """
     Return path to existing directory in which jobs put their files and cache
 
     :param str content_path: Path to torrent content
+    :param str base: Location of the project directory; defaults to
+        :attr:`~.constants.CACHE_DIRPATH`
 
-    :raise RuntimeError: if `content_path` exists and is not a directory or has
+    :raise ContentError: if `content_path` exists and is not a directory or has
         insufficient permissions
     """
     if content_path:
-        path = basename(content_path)
-        path += '.upsies'
+        if not base:
+            base = constants.CACHE_DIRPATH
+        path = os.path.join(base, basename(content_path) + '.upsies')
     else:
         path = '.'
     if not os.path.exists(path):
-        os.mkdir(path)
-    assert_dir_usable(path)
+        mkdir(path)
     _log.debug('Using project directory: %r', path)
     return path
 
 
+def limit_directory_size(path, max_total_size):
+    """
+    Delete oldest files (by access time) until maximum size is not exceeded
+
+    :func:`prune_empty_directories` is also called with `path`.
+
+    :param path: Path to directory
+    :param max_total_size: Maximum combined size of all files in `path` and its
+        subdirectories
+    """
+    def size(path):
+        return sum(file_size(f) for f in file_list(path))
+
+    # This should return mtime if file system was mounted with noatime.
+    def atime(filepath):
+        statinfo = os.stat(filepath)
+        return statinfo.st_atime
+
+    # Remove oldest file until `path` size is small enough
+    while size(path) > max_total_size:
+        files = sorted(file_list(path), key=atime)
+        if files:
+            try:
+                os.unlink(files[0])
+            except OSError as e:
+                if e.strerror:
+                    raise RuntimeError(f'{files[0]}: Failed to prune: {e.strerror}')
+                else:
+                    raise RuntimeError(f'{files[0]}: Failed to prune: {e}')
+                break
+
+    prune_empty_directories(path)
+
+
+def prune_empty_directories(path):
+    """Remove empty subdirectories recursively"""
+    try:
+        for dirpath, dirnames, _ in os.walk(path, topdown=False):
+            for dirname in dirnames:
+                subdirpath = os.path.join(dirpath, dirname)
+                if not os.listdir(subdirpath):
+                    os.rmdir(subdirpath)
+        if not os.listdir(path):
+            os.rmdir(path)
+    except OSError as e:
+        if e.strerror:
+            raise RuntimeError(f'{subdirpath}: Failed to prune: {e.strerror}')
+        else:
+            raise RuntimeError(f'{subdirpath}: Failed to prune: {e}')
+
+
+def mkdir(path):
+    """
+    Create directory and its parents
+
+    Existing directories are ignored.
+
+    :raise ContentError: if directory creation fails
+    """
+    try:
+        os.makedirs(path, exist_ok=True)
+    except OSError as e:
+        if e.strerror:
+            raise errors.ContentError(f'{path}: {e.strerror}')
+        else:
+            raise errors.ContentError(f'{path}: {e}')
+    else:
+        assert_dir_usable(path)
+
+
 def basename(path):
     """
     Return last segment in `path`
@@ -106,6 +163,17 @@ def dirname(path):
     return os.path.dirname(str(path).rstrip(os.sep))
 
 
+def file_and_parent(path):
+    """Return `path`'s filename and parent directoy name if there is one"""
+    if os.sep in path:
+        return (
+            basename(path),
+            basename(dirname(path)),
+        )
+    else:
+        return (path,)
+
+
 def sanitize_filename(filename):
     """
     Replace illegal characters in `filename` with "_"
@@ -120,22 +188,61 @@ def sanitize_filename(filename):
 
 def file_extension(path):
     """
-    Extract file extension
+    Return file extension
 
     Unlike :func:`os.path.splitext`, this function expects the extension to be 1-3
     characters long and consist only of ascii characters or numbers.
 
-    :param str path: Path to file
+    :param str path: Path to file or directory
 
     :return: file extension
     :rtype: str
     """
     path = str(path)
     if '.' in path:
-        return re.sub(r'^.*?\.([a-zA-Z0-9]{1,3})$', r'\1', path).lower()
+        return re.sub(r'^.*\.([a-zA-Z0-9]{1,3})$', r'\1', path).lower()
     else:
         return ''
 
+def strip_extension(path, only=()):
+    """
+    Return `path` without file extension
+
+    If `path` doesn't have a file extension, return it as is.
+
+    A file extension is 1-3 characters long and consist only of ascii characters
+    or numbers.
+
+    :param str path: Path to file or directory
+    :param only: Only strip extension if it exists in this sequence
+    :type only: sequence of :class:`str`
+
+    :return: file extension
+    :rtype: str
+    """
+    path = str(path)
+    if '.' in path:
+        match = re.search(r'^(.*)\.([a-zA-Z0-9]{1,3})$', path)
+        if match:
+            name = match.group(1)
+            ext = match.group(2)
+            if only:
+                if ext in only:
+                    return name
+            else:
+                return name
+    return path
+
+
+def file_size(path):
+    """Return file size in bytes or `None`"""
+    if not os.path.isdir(path):
+        try:
+            return os.path.getsize(path)
+        except OSError:
+            pass
+    return None
+
 
 def file_list(path, extensions=()):
     """
@@ -180,7 +287,7 @@ _CORNER     = '\u2514'  # noqa: E221  # └
 
 def file_tree(tree, _parents_is_last=()):
     """
-    Create a pretty file tree
+    Create a pretty file tree as multi-line string
 
     :param tree: Nested 2-tuples: The first item is the file or directory name,
         the second item is the file size for files or a tuple for directories
@@ -214,7 +321,7 @@ def file_tree(tree, _parents_is_last=()):
                 indent += f'{_DOWN_RIGHT}{_RIGHT}'
 
         if isinstance(node, int):
-            lines.append(f'{indent}{name} ({pretty_bytes(node)})')
+            lines.append(f'{indent}{name} ({string.pretty_bytes(node)})')
         else:
             lines.append(f'{indent}{name}')
             # Descend into child node
