@@ -1,9 +1,8 @@
 """
-HTTP methods
+HTTP methods with caching
 """
 
 import asyncio
-import atexit
 import collections
 import hashlib
 import json
@@ -11,7 +10,7 @@ import os
 
 import httpx
 
-from .. import __project_name__, __version__, errors
+from .. import __project_name__, __version__, constants, errors
 from . import fs
 
 import logging  # isort:skip
@@ -27,14 +26,23 @@ _default_headers = {
 _request_locks = collections.defaultdict(lambda: asyncio.Lock())
 
 _client = httpx.AsyncClient(
-    timeout=30,
+    timeout=60,
 )
 
-def _close_client():
+
+cache_directory = None
+"""
+Where to store cached requests
+
+If this is set to a falsy value, default to :attr:`~.constants.CACHE_DIRPATH`.
+"""
+
+
+def close():
+    """Close the client session"""
     _log.debug('Closing client: %r', _client)
     asyncio.get_event_loop().run_until_complete(_client.aclose())
     _log.debug('Closed client: %r', _client)
-atexit.register(_close_client)
 
 
 async def get(url, headers={}, params={}, auth=None,
@@ -97,16 +105,48 @@ async def post(url, headers={}, data={}, files={}, auth=None,
         allow_redirects=allow_redirects,
     )
 
+async def download(url, filepath, *args, **kwargs):
+    """
+    Write downloaded data to file
 
-class Response(str):
+    :param url: Where to download the data from
+    :param filepath: Where to save the downloaded data
+
+    Any other arguments are passed to :func:`get`, except for `cache`, which is
+    always `False`.
+
+    If `filepath` exists, no request is made.
+
+    :raise RequestError: if anything goes wrong
+    :return: `filepath`
+    """
+    kwargs['cache'] = False
+    if not os.path.exists(filepath):
+        _log.debug('Downloading %r to %r', url, filepath)
+        response = await get(url, *args, **kwargs)
+        try:
+            with open(filepath, 'wb') as f:
+                f.write(response.bytes)
+        except OSError as e:
+            if e.strerror:
+                raise errors.RequestError(f'Unable to write {filepath}: {e.strerror}')
+            else:
+                raise errors.RequestError(f'Unable to write {filepath}: {e}')
+    else:
+        _log.debug('Already downloaded %r to %r', url, filepath)
+    return filepath
+
+
+class Result(str):
     """
     Response to an HTTP request
 
     This is a subclass of :class:`str` with additional attributes and methods.
     """
 
-    def __new__(cls, text, headers={}, status_code=None):
+    def __new__(cls, text, bytes, headers={}, status_code=None):
         obj = super().__new__(cls, text)
+        obj._bytes = bytes
         obj._headers = headers
         obj._status_code = status_code
         return obj
@@ -121,6 +161,11 @@ class Response(str):
         """HTTP status code"""
         return self._status_code
 
+    @property
+    def bytes(self):
+        """Response data as :class:`bytes`"""
+        return self._bytes
+
     def json(self):
         """
         Parse the response text as JSON
@@ -136,6 +181,7 @@ class Response(str):
     def __repr__(self):
         return (f'{type(self).__name__}('
                 f'text={str(self)!r}, '
+                f'bytes={self.bytes!r}, '
                 f'headers={self.headers!r}, '
                 f'status_code={self.status_code!r})')
 
@@ -146,14 +192,19 @@ async def _request(method, url, headers={}, params={}, data={}, files={},
     if method.upper() not in ('GET', 'POST'):
         raise ValueError(f'Invalid method: {method}')
 
+    if isinstance(data, (bytes, str)):
+        payload = {'content': data}
+    else:
+        payload = {'data': data}
+
     headers = {**_default_headers, **headers}
     request = _client.build_request(
         method=str(method),
         headers=headers,
         url=str(url),
         params=params,
-        data=data,
         files=_open_files(files),
+        **payload,
     )
 
     if not user_agent:
@@ -161,14 +212,14 @@ async def _request(method, url, headers={}, params={}, data={}, files={},
 
     # Block when requesting the same URL simultaneously
     request_lock_key = (request.url, await request.aread())
-    _log.debug('Request lock key: %r', request_lock_key)
+    # _log.debug('Request lock key: %r', request_lock_key)
     request_lock = _request_locks[request_lock_key]
     async with request_lock:
         if cache:
             cache_file = _cache_file(method, url, params)
-            response = _from_cache(cache_file)
-            if response is not None:
-                return Response(response)
+            result = _from_cache(cache_file)
+            if result is not None:
+                return result
 
         _log.debug('%s: %r: %r', method, url, params)
         try:
@@ -185,16 +236,18 @@ async def _request(method, url, headers={}, params={}, data={}, files={},
                     headers=response.headers,
                     status_code=response.status_code,
                 )
-        except httpx.NetworkError:
-            raise errors.RequestError(f'{url}: Failed to connect')
+        except httpx.TimeoutException:
+            raise errors.RequestError(f'{url}: Timeout')
         except httpx.HTTPError as e:
+            _log.debug(f'Unexpected HTTP error: {e!r}')
             raise errors.RequestError(f'{url}: {e}')
         else:
             if cache:
                 cache_file = _cache_file(method, url, params)
-                _to_cache(cache_file, response.text)
-            return Response(
+                _to_cache(cache_file, response.content)
+            return Result(
                 text=response.text,
+                bytes=response.content,
                 headers=response.headers,
                 status_code=response.status_code,
             )
@@ -235,6 +288,7 @@ def _open_files(files):
             raise RuntimeError(f'Invalid file: {fileinfo}')
     return opened
 
+
 def _get_fileobj(filepath):
     """
     Open `filepath` and return the file object
@@ -251,22 +305,36 @@ def _get_fileobj(filepath):
         raise errors.RequestError(f'{filepath}: {msg}')
 
 
-def _from_cache(cache_file):
+def _to_cache(cache_file, bytes):
     try:
-        with open(cache_file, 'r') as f:
-            data = f.read()
-    except OSError:
-        return None
-    else:
-        return data
-
-def _to_cache(cache_file, string):
-    try:
-        with open(cache_file, 'w') as f:
-            f.write(string)
+        fs.mkdir(fs.dirname(cache_file))
+        with open(cache_file, 'wb') as f:
+            f.write(bytes)
     except OSError as e:
         raise RuntimeError(f'Unable to write cache file {cache_file}: {e}')
 
+
+def _from_cache(cache_file):
+    bytes = _read_bytes_from_file(cache_file)
+    text = _read_string_from_file(cache_file)
+    if bytes and text:
+        return Result(text=text, bytes=bytes)
+
+
+def _read_string_from_file(filepath):
+    bytes = _read_bytes_from_file(filepath)
+    if bytes:
+        return str(bytes, encoding='utf-8', errors='replace')
+
+
+def _read_bytes_from_file(filepath):
+    try:
+        with open(filepath, 'rb') as f:
+            return f.read()
+    except OSError:
+        pass
+
+
 def _cache_file(method, url, params={}):
     def make_filename(method, url, params_str):
         if params_str:
@@ -283,10 +351,11 @@ def _cache_file(method, url, params={}):
         params_str = ''
 
     return os.path.join(
-        fs.tmpdir(),
+        cache_directory or constants.CACHE_DIRPATH,
         make_filename(method, url, params_str),
     )
 
+
 def _semantic_hash(obj):
     """
     Return hash for `obj` that stays the same between sessions
