@@ -42,10 +42,36 @@ class WebDbApiBase(abc.ABC):
     async def countries(self, id):
         """Return list of country names"""
 
+    @abc.abstractmethod
+    async def creators(self, id):
+        """Return list of creator names (usually empty for movies and episodes)"""
+
+    @abc.abstractmethod
+    async def directors(self, id):
+        """Return list of director names (usually empty for series)"""
+
     @abc.abstractmethod
     async def keywords(self, id):
         """Return list of keywords, e.g. genres"""
 
+    @abc.abstractmethod
+    async def poster_url(self, id):
+        """Return URL of poster image or `None`"""
+
+    @abc.abstractmethod
+    async def rating(self, id):
+        """Return rating as a number or `None`"""
+
+    @property
+    @abc.abstractmethod
+    async def rating_min(self):
+        """Minimum :meth:`rating` value"""
+
+    @property
+    @abc.abstractmethod
+    async def rating_max(self):
+        """Maximum :meth:`rating` value"""
+
     @abc.abstractmethod
     async def summary(self, id):
         """Return short plot description"""
@@ -60,7 +86,11 @@ class WebDbApiBase(abc.ABC):
 
     @abc.abstractmethod
     async def type(self, id):
-        """Return :class:`~.utils.ReleaseType`"""
+        """Return :class:`~.types.ReleaseType`"""
+
+    @abc.abstractmethod
+    async def url(self, id):
+        """Return URL for `id`"""
 
     @abc.abstractmethod
     async def year(self, id):
@@ -79,6 +109,8 @@ class WebDbApiBase(abc.ABC):
         corofuncs = (getattr(self, method) for method in methods)
         awaitables = (corofunc(id) for corofunc in corofuncs)
         results = await asyncio.gather(*awaitables)
+        dct = {'id': id}
         # "The order of result values corresponds to the order of awaitables in `aws`."
         # https://docs.python.org/3/library/asyncio-task.html#running-tasks-concurrently
-        return {method: result for method, result in zip(methods, results)}
+        dct.update((method, result) for method, result in zip(methods, results))
+        return dct
