@@ -5,7 +5,8 @@ subclasses
 
 import re
 
-from .. import ReleaseType, release
+from .. import release
+from ..types import ReleaseType
 
 import logging  # isort:skip
 _log = logging.getLogger(__name__)
@@ -16,7 +17,7 @@ class Query:
     Search query for databases like IMDb
 
     :param str title: Name of the movie or TV series
-    :param str type: :class:`~.utils.ReleaseType` enum or one of its value names
+    :param str type: :class:`~.types.ReleaseType` enum or one of its value names
     :param year: Year of release
     :type year: str or int
 
@@ -37,7 +38,7 @@ class Query:
 
     @property
     def type(self):
-        """:class:`~.utils.ReleaseType` value"""
+        """:class:`~.types.ReleaseType` value"""
         return self._type
 
     @property
@@ -57,9 +58,8 @@ class Query:
 
     _types = {
         ReleaseType.movie: ('movie', 'film'),
-        ReleaseType.season: ('season',),
+        ReleaseType.season: ('season', 'series', 'tv', 'show', 'tvshow'),
         ReleaseType.episode: ('episode',),
-        ReleaseType.series: ('series', 'tv', 'show', 'tvshow'),
     }
     _known_types = tuple(v for types in _types.values() for v in types)
     _kw_regex = {
@@ -92,8 +92,6 @@ class Query:
                     value = match.group(1)
                     if kw == 'type' and value in cls._types[ReleaseType.movie]:
                         return 'type', ReleaseType.movie
-                    elif kw == 'type' and value in cls._types[ReleaseType.series]:
-                        return 'type', ReleaseType.series
                     elif kw == 'type' and value in cls._types[ReleaseType.season]:
                         return 'type', ReleaseType.season
                     elif kw == 'type' and value in cls._types[ReleaseType.episode]:
@@ -122,12 +120,12 @@ class Query:
         `path` is passed to :class:`~.release.ReleaseInfo` to get the
         arguments for instantiation.
         """
-        guess = release.ReleaseInfo(path)
-        kwargs = {'title': guess['title']}
-        if guess.get('year'):
-            kwargs['year'] = guess['year']
-        if guess.get('type'):
-            kwargs['type'] = guess['type']
+        info = release.ReleaseInfo(path)
+        kwargs = {'title': info['title']}
+        if info.get('year'):
+            kwargs['year'] = info['year']
+        if info.get('type'):
+            kwargs['type'] = info['type']
         return cls(**kwargs)
 
     def __eq__(self, other):
@@ -179,7 +177,7 @@ class SearchResult:
     :param str title: Title of the movie or series
     :param str title_english: English title of the movie or series
     :param str title_original: Original title of the movie or series
-    :param str type: :class:`~.utils.ReleaseType` value
+    :param str type: :class:`~.types.ReleaseType` value
     :param str url: Web page of the search result
     :param str year: Release year; for series this should be the year of the
         first airing of the first episode of the first season
@@ -212,3 +210,12 @@ class SearchResult:
     def __repr__(self):
         kwargs = ', '.join(f'{k}={v!r}' for k, v in self._info.items())
         return f'{type(self).__name__}({kwargs})'
+
+
+class Person(str):
+    """:class:`str` subclass with an `url` attribute"""
+
+    def __new__(cls, string, url=''):
+        obj = super().__new__(cls, string)
+        obj.url = url
+        return obj
